/*
 * Simple ISAKMP transparent proxy for in-kernel use.  For use with the NAT
 * code.
 *
 * $Id: ip_ipsec_pxy.c,v 1.1.2.4 2001/10/23 12:53:13 darrenr Exp $
 *
 */
#define	IPF_IPSEC_PROXY


int ippr_ipsec_init __P((void));
int ippr_ipsec_new __P((fr_info_t *, ip_t *, ap_session_t *, nat_t *));
int ippr_ipsec_out __P((fr_info_t *, ip_t *, ap_session_t *, nat_t *));

static	frentry_t	ipsecfr;


/*
 * RCMD application proxy initialization.
 */
int ippr_ipsec_init()
{
	bzero((char *)&ipsecfr, sizeof(ipsecfr));
	ipsecfr.fr_ref = 1;
	ipsecfr.fr_flags = FR_OUTQUE|FR_PASS|FR_QUICK|FR_KEEPSTATE;
	return 0;
}


/*
 * Setup for a new IPSEC proxy.
 */
int ippr_ipsec_new(fin, ip, aps, nat)
fr_info_t *fin;
ip_t *ip;
ap_session_t *aps;
nat_t *nat;
{
	ipsec_pxy_t *ipsec;
	fr_info_t fi;
	ipnat_t *ipn;
	int p;

	/*
	 * Because _new() gets called from nat_new(), ipf_nat is held with a
	 * write lock so pass rw=1 to nat_outlookup().
	 */
	if (nat_outlookup(fin->fin_ifp, 0, IPPROTO_ESP, nat->nat_inip,
			  ip->ip_dst, 0, 1) != NULL)
		return -1;

	aps->aps_psiz = sizeof(*ipsec);
	KMALLOCS(aps->aps_data, ipsec_pxy_t *, sizeof(*ipsec));
	if (aps->aps_data == NULL)
		return -1;

	ipsec = aps->aps_data;
	bzero((char *)ipsec, sizeof(*ipsec));

	ipsec->ipsc_spi = *(u_32_t *)fin->fin_dp;

	/*
	 * Create rule against which the tunnel/transport mapping is created.
	 */
	ipn = &ipsec->ipsc_rule;
	ipn->in_ifp = fin->fin_ifp;
	ipn->in_apr = NULL;
	ipn->in_use = 1;
	ipn->in_hits = 1;
	ipn->in_nip = nat->nat_outip.s_addr;
	ipn->in_ippip = 1;
	ipn->in_inip = nat->nat_inip.s_addr;
	ipn->in_inmsk = 0xffffffff;
	ipn->in_outip = fin->fin_saddr;
	ipn->in_outmsk = nat->nat_outip.s_addr;
	ipn->in_srcip = fin->fin_saddr;
	ipn->in_srcmsk = 0xffffffff;
	ipn->in_redir = NAT_MAP;
	bcopy(nat->nat_ptr->in_ifname, ipn->in_ifname, sizeof(ipn->in_ifname));
	ipn->in_p = IPPROTO_ESP;

	bcopy((char *)fin, (char *)&fi, sizeof(fi));
	fi.fin_fi.fi_p = IPPROTO_ESP;
	fi.fin_fr = &ipsecfr;
	p = ip->ip_p;
	ip->ip_p = IPPROTO_ESP;

	ipsec->ipsc_nat = nat_new(ipn, ip, &fi, 0, NAT_OUTBOUND);
	if (ipsec->ipsc_nat != NULL) {
		ipsec->ipsc_state = fr_addstate(ip, &fi, 0);
	}
	ip->ip_p = p;
	return 0;
}


int ippr_ipsec_out(fin, ip, aps, nat)
fr_info_t *fin;
ip_t *ip;
ap_session_t *aps;
nat_t *nat;
{
	ipsec_pxy_t *ipsec;

	ipsec = aps->aps_data;
	if (ipsec != NULL) {
		if (ipsec->ipsc_nat != NULL)
			ipsec->ipsc_nat->nat_age = nat->nat_age;
		if (ipsec->ipsc_state != NULL)
			ipsec->ipsc_state->is_age = nat->nat_age;
	}
	return 0;
}
